<?php
/**
 * Loads Digraphs library.
 *
 * See
 * https://github.com/danvk/dygraphs/blob/master/src/dygraph-options-reference.js
 *
 * @author Mario De Weerd
 */
class YDygraphs extends CWidget {

    public $data;
    /* When true, use the official content network */
    public $useCDN=true;
    
    /**
     * Provide id to synchronize between chartgs.
     * 
     * Loads synchronizer
     */
    public $synchronizerId;
    /**
     * When true, add unzoom function
     */
    public $unzoom;

    public $cdnUrl ='//cdnjs.cloudflare.com/ajax/libs/dygraph/1.1.1/dygraph-combined.js';

    /** List of extra assets */
    public $jsFiles = array();
    public $cssFiles = array();
    
    public $htmlOptions=array();
    /**
     * Update event.
     *
     * Binds the widget to this update event.
     * Expects parameter to be new value.
     */
    public $updateEvent;
    /**
     * Selector to select item on which the update event must be bound.
     */
    public $updateBindSelector="body";
    /**
     * Which element to use from the event data.
     * @var string
     */
    public $updateData='data';//data[data.length-1] for last element of array.
    
    /**
     * When true, cache the code. Default: true
     */
    public $cache=false;
    
    /**
     * cacheid, used for caching. Default: "default"
     */
    public $cacheid="default";
    
    /**
     * Expire time in seconds for cache. Default: 3600
     */
    public $cacheexpire=3600;
    
    /**
     * (non-PHPdoc)
     * @see CWidget::init()
     */
    public function init() {
        /* Call the initializing function */
        $this->htmlOptions['id']=$this->getId();
        $this->_init();
    }


    /**
     * Initialize the widget - include assets in the web page when debugging.
     */
    private function _init() {
       $am=Yii::app()->assetManager;
       $path = $am->publish(__DIR__.DIRECTORY_SEPARATOR."assets",false,-1/*,YII_DEBUG*/);
       $cs=Yii::app()->clientScript;
        
        if(!$this->useCDN) {
            $jsFiles=array(YII_DEBUG?'dygraph-combined-dev.js':'dygraph-combined.js',);
            foreach($jsFiles as $j) {
                $cs->registerScriptFile("$path/js/$j");
            }
        } else {
            $cs->registerScriptFile($this->cdnUrl);
        }
        foreach($this->jsFiles as $asset) {
            $cs->registerScriptFile("$path/js/$asset");
        }
        if(isset($this->synchronizerId)) {
            $cs->registerScriptFile("$path/js/extras/synchronizer.js");
        }
        if(isset($this->unzoom)&&$this->unzoom) {
            $cs->registerScriptFile("$path/js/extras/unzoom.js");
        }
        foreach($this->cssFiles as $f) {
        	$cs->registerCssFile("$path/css/$f");
        }
        echo CHtml::tag('div',$this->htmlOptions,'');
    }

    /**
     * (non-PHPdoc)
     * @see CWidget::run()
     */
    public function run() {
        $cs=Yii::app()->getClientScript();
        $js=false;
        
        if(isset($this->synchronizerId)) {
            $syncVar="window._dygraphsync.".$this->synchronizerId;
            $syncJs="window._dygraphsync||(window._dygraphsync=[]);$syncVar||($syncVar=[]);$syncVar.push(g);";
        } else {
            $syncJs="";
        }
        
        $cacheid=__CLASS__.Yii::app()->language.$this->cacheid.$this->id;
        
        if($this->cache) {
            $js=Yii::app()->cache->get($cacheid);
        }
        if($js === false) {
            $optionList=array(
                'xValueParser',
                'stackedGraph',
                'stackedGraphNaNFill',
                'pointSize',
                'drawPoints',
                'drawGapEdgePoints',
                'height',
                'color',
                'colors',
                'connectSeparatedPoints',
                'highlightSeriesOpts',
                'highlightSeriesBackgroundAlpha',
                'highlightSeriesBackgroundColor',
                'includeZero',
                'rollPeriod',
                'axisTickSize',
                'labelsSeparateLines',
                'valueFormatter',
                'labelsKMG2',
                'delimiter',
                'axisLabelFontSize',
                'width',
                'interactionModel',
                'ticker',
                'xAxisHeight',
                'showLabelsOnHighlight',
                'axis',
                'pixelsPerLabel',
                'labelsDiv',
                'fractions',
                'logscale',
                'strokeWidth',
                'strokePattern',
                'strokeBorderWidth',
                'strokeBorderColor',
                'wilsonInterval',
                'fillGraph',
                'highlightCircleSize',
                'gridLineColor',
                'gridLinePattern',
                'visibility',
                'valueRange',
                'colorSaturation',
                'hideOverlayOnMouseOut',
                'legend',
                'legendFormatter',
                'labelsShowZeroValues',
                'stepPlot',
                'labelsUTC',
                'labelsKMB',
                'rightGap',
                'avoidMinZero',
                'drawAxesAtZero',
                'xRangePad',
                'yRangePad',
                'axisLabelFormatter',
                'labels',
                'dateWindow',
                'showRoller',
                'sigma',
                'customBars',
                'colorValue',
                'errorBars',
                'displayAnnotations',
                'panEdgeFraction',
                'title',
                'titleHeight',
                'xlabel',
                'xLabelHeight',
                'ylabel',
                'y2label',
                'yLabelWidth',
                'isZoomedIgnoreProgrammaticZoom',
                'drawGrid',
                'independentTicks',
                'drawAxis',
                'gridLineWidth',
                'axisLineWidth',
                'axisLineColor',
                'fillAlpha',
                'axisLabelWidth',
                'sigFigs',
                'digitsAfterDecimal',
                'maxNumberWidth',
                'file',
                'timingName',
                'showRangeSelector',
                'rangeSelectorHeight',
                'rangeSelectorPlotStrokeColor',
                'rangeSelectorPlotFillColor',
                'rangeSelectorPlotFillGradientColor',
                'rangeSelectorBackgroundStrokeColor',
                'rangeSelectorBackgroundLineWidth',
                'rangeSelectorPlotLineWidth',
                'rangeSelectorForegroundStrokeColor',
                'rangeSelectorForegroundLineWidth',
                'rangeSelectorAlpha',
                'showInRangeSelector',
                'animatedZooms',
                'plotter',
                'axes',
                'series',
                'plugins',
            );
            
            $callbacks=array(
                'drawPointCallback',
                'zoomCallback',
                'pointClickCallback',
                'highlightCallback',
                'drawHighlightPointCallback',
                'unhighlightCallback',
                'annotationMouseOverHandler',
                'annotationMouseOutHandler',
                'annotationClickHandler',
                'annotationDblClickHandler',
                'drawCallback',
                'underlayCallback',
                'clickCallback',
                'dataHandler',
            );
        
            $options=array();
            foreach($optionList as $option) {
                if(isset($this->$option)) {
                    $options[$option]=$this->$option;
                }
            }
        
            // JavaScript options to embed in function.
            foreach($callbacks as $option) {
                if("{$this->$option}"!="") {
                    $options[$option]=new CJavaScriptExpression("function(values){{$this->$option}}");
                }
            }
            
            if(isset($this->unzoom)&&$this->unzoom) {
                if(!isset($options['plugins'])) {
                    $options['plugins']=array();
                }
                $options['plugins'][]=new CJavaScriptExpression('Dygraph.Plugins.Unzoom');
            }
            
            if(count($options)) {
                $jsOpt=CJavaScript::encode($options);
            } else {
                $jsOpt='';
            }
        
            // Example code - in case of locale.
            // $localeExtend=array('locale'=>array(
            //   'addAll'=>Yii::t('YMultiSelect.t','Add all'),
            //   'removeAll'=>Yii::t('YMultiSelect.t','Remove all'),
            //   'itemsCount'=>Yii::t('YMultiSelect.t','items selected'),
            // ));
            //$js='jQuery.extend(jQuery.ui.multiselect,'.JavaScript::encode($localeExtend).');';
            $dataJs=CJavaScript::encode($this->data);
            $js="(function(){var g=new Dygraph(document.getElementById(\"{$this->id}\"),$dataJs,$jsOpt);$syncJs})();";
            Yii::app()->cache->set($cacheid, $js, $this->cacheexpire);
        }
        $cs->registerScript($cacheid,$js,CClientScript::POS_END);
        if(isset($this->synchronizerId)) {
            $jsSynchronizeId="Dygraph.synchronize($syncVar);";
            $this->registerDygraphScript(__CLASS__."#".$jsSynchronizeId,$jsSynchronizeId,CClientScript::POS_READY);
        }

        $this->registerDygraphScript($cacheid, $js, CClientScript::POS_END);
    }
   
    static private $DiGraphId=0;
   	public function registerDygraphScript($id,$script,$position=null,array $htmlOptions=array())
	{
   		self::$DiGraphId++;
   		$sid="dig_".self::$DiGraphId;
		$js="var {$sid}=function(){if(typeof(Dygraph)==='undefined'){window.setTimeout({$sid},5);}else{{$script}};};{$sid}();";
		$cs=Yii::app()->clientScript;
		$cs->registerScript($id,$js,$position,$htmlOptions);
   	}
   	
   	/**
   	 * A function which parses x-values (i.e. the dependent series). Must return a number, even when the values are dates. In this case, millis since epoch are used. This is used primarily for parsing CSV data. *=Dygraphs is slightly more accepting in the dates which it will parse. See code for details.
   	 *
   	 * @var function(str) -> number
   	 */
   	public $xValueParser;
   	
   	/**
   	 * If set, stack series on top of one another rather than drawing them independently. The first series specified in the input data will wind up on top of the chart and the last will be on bottom. NaN values are drawn as white areas without a line on top, see stackedGraphNaNFill for details.
   	 *
   	 * @var boolean
   	 */
   	public $stackedGraph;
   	
   	/**
   	 * Controls handling of NaN values inside a stacked graph. NaN values are interpolated/extended for stacking purposes, but the actual point value remains NaN in the legend display. Valid option values are "all" (interpolate internally, repeat leftmost and rightmost value as needed), "inside" (interpolate internally only, use zero outside leftmost and rightmost value), and "none" (treat NaN as zero everywhere).
   	 *
   	 * @var string
   	 */
   	public $stackedGraphNaNFill;
   	
   	/**
   	 * The size of the dot to draw on each point in pixels (see drawPoints). A dot is always drawn when a point is "isolated", i.e. there is a missing point on either side of it. This also controls the size of those dots.
   	 *
   	 * @var integer
   	 */
   	public $pointSize;
   	
   	/**
   	 * Draw a small dot at each point, in addition to a line going through the point. This makes the individual data points easier to see, but can increase visual clutter in the chart. The small dot can be replaced with a custom rendering by supplying a <a href='#drawPointCallback'>drawPointCallback</a>.
   	 *
   	 * @var boolean
   	 */
   	public $drawPoints;
   	
   	/**
   	 * Draw points at the edges of gaps in the data. This improves visibility of small data segments or other data irregularities.
   	 *
   	 * @var boolean
   	 */
   	public $drawGapEdgePoints;
   	
   	/**
   	 * Draw a custom item when drawPoints is enabled. Default is a small dot matching the series color. This method should constrain drawing to within pointSize pixels from (cx, cy).  Also see <a href='#drawHighlightPointCallback'>drawHighlightPointCallback</a>
   	 *
   	 * @var function(g, seriesName, canvasContext, cx, cy, color, pointSize)
   	 */
   	public $drawPointCallback;
   	
   	/**
   	 * Height, in pixels, of the chart. If the container div has been explicitly sized, this will be ignored.
   	 *
   	 * @var integer
   	 */
   	public $height;
   	
   	/**
   	 * A function to call when the zoom window is changed (either by zooming in or out). When animatedZooms is set, zoomCallback is called once at the end of the transition (it will not be called for intermediate frames).
   	 *
   	 * @var function(minDate, maxDate, yRanges)
   	 */
   	public $zoomCallback;
   	
   	/**
   	 * A function to call when a data point is clicked. and the point that was clicked.
   	 *
   	 * @var function(e, point)
   	 */
   	public $pointClickCallback;
   	
   	/**
   	 * A per-series color definition. Used in conjunction with, and overrides, the colors option.
   	 *
   	 * @var string
   	 */
   	public $color;
   	
   	/**
   	 * List of colors for the data series. These can be of the form "#AABBCC" or "rgb(255,100,200)" or "yellow", etc. If not specified, equally-spaced points around a color wheel are used. Overridden by the 'color' option.
   	 *
   	 * @var array<string>
   	 */
   	public $colors;
   	
   	/**
   	 * Usually, when Dygraphs encounters a missing value in a data series, it interprets this as a gap and draws it as such. If, instead, the missing values represents an x-value for which only a different series has data, then you'll want to connect the dots by setting this to true. To explicitly include a gap with this option set, use a value of NaN.
   	 *
   	 * @var boolean
   	 */
   	public $connectSeparatedPoints;
   	
   	/**
   	 * When set, this callback gets called every time a new point is highlighted.
   	 *
   	 * @var function(event, x, points, row, seriesName)
   	 */
   	public $highlightCallback;
   	
   	/**
   	 * Draw a custom item when a point is highlighted.  Default is a small dot matching the series color. This method should constrain drawing to within pointSize pixels from (cx, cy) Also see <a href='#drawPointCallback'>drawPointCallback</a>
   	 *
   	 * @var function(g, seriesName, canvasContext, cx, cy, color, pointSize)
   	 */
   	public $drawHighlightPointCallback;
   	
   	/**
   	 * When set, the options from this object are applied to the timeseries closest to the mouse pointer for interactive highlighting. See also 'highlightCallback'. Example: highlightSeriesOpts: { strokeWidth: 3 }.
   	 *
   	 * @var Object
   	 */
   	public $highlightSeriesOpts;
   	
   	/**
   	 * Fade the background while highlighting series. 1=fully visible background (disable fading), 0=hiddden background (show highlighted series only).
   	 *
   	 * @var float
   	 */
   	public $highlightSeriesBackgroundAlpha;
   	
   	/**
   	 * Sets the background color used to fade out the series in conjunction with 'highlightSeriesBackgroundAlpha'.
   	 *
   	 * @var string
   	 */
   	public $highlightSeriesBackgroundColor;
   	
   	/**
   	 * Usually, dygraphs will use the range of the data plus some padding to set the range of the y-axis. If this option is set, the y-axis will always include zero, typically as the lowest value. This can be used to avoid exaggerating the variance in the data
   	 *
   	 * @var boolean
   	 */
   	public $includeZero;
   	
   	/**
   	 * Number of days over which to average data. Discussed extensively above.
   	 *
   	 * @var integer &gt;= 1
   	 */
   	public $rollPeriod;
   	
   	/**
   	 * When set, this callback gets called every time the user stops highlighting any point by mousing out of the graph.
   	 *
   	 * @var function(event)
   	 */
   	public $unhighlightCallback;
   	
   	/**
   	 * The size of the line to display next to each tick mark on x- or y-axes.
   	 *
   	 * @var number
   	 */
   	public $axisTickSize;
   	
   	/**
   	 * Put <code>&lt;br/&gt;</code> between lines in the label string. Often used in conjunction with <strong>labelsDiv</strong>.
   	 *
   	 * @var boolean
   	 */
   	public $labelsSeparateLines;
   	
   	/**
   	 * Function to provide a custom display format for the values displayed on mouseover. This does not affect the values that appear on tick marks next to the axes. To format those, see axisLabelFormatter. This is usually set on a <a href='per-axis.html'>per-axis</a> basis. .
   	 *
   	 * @var function(num or millis, opts, seriesName, dygraph, row, col)
   	 */
   	public $valueFormatter;
   	
   	/**
   	 * If provided, this function is called whenever the user mouses over an annotation.
   	 *
   	 * @var function(annotation, point, dygraph, event)
   	 */
   	public $annotationMouseOverHandler;
   	
   	/**
   	 * If provided, this function is called whenever the user mouses out of an annotation.
   	 *
   	 * @var function(annotation, point, dygraph, event)
   	 */
   	public $annotationMouseOutHandler;
   	
   	/**
   	 * If provided, this function is called whenever the user clicks on an annotation.
   	 *
   	 * @var function(annotation, point, dygraph, event)
   	 */
   	public $annotationClickHandler;
   	
   	/**
   	 * If provided, this function is called whenever the user double-clicks on an annotation.
   	 *
   	 * @var function(annotation, point, dygraph, event)
   	 */
   	public $annotationDblClickHandler;
   	
   	/**
   	 * When set, this callback gets called every time the dygraph is drawn. This includes the initial draw, after zooming and repeatedly while panning.
   	 *
   	 * @var function(dygraph, is_initial)
   	 */
   	public $drawCallback;
   	
   	/**
   	 * Show k/M/G for kilo/Mega/Giga on y-axis. This is different than <code>labelsKMB</code> in that it uses base 2, not 10.
   	 *
   	 * @var boolean
   	 */
   	public $labelsKMG2;
   	
   	/**
   	 * The delimiter to look for when separating fields of a CSV file. Setting this to a tab is not usually necessary, since tab-delimited data is auto-detected.
   	 *
   	 * @var string
   	 */
   	public $delimiter;
   	
   	/**
   	 * Size of the font (in pixels) to use in the axis labels, both x- and y-axis.
   	 *
   	 * @var integer
   	 */
   	public $axisLabelFontSize;
   	
   	/**
   	 * When set, this callback gets called before the chart is drawn. It details on how to use this.
   	 *
   	 * @var function(context, area, dygraph)
   	 */
   	public $underlayCallback;
   	
   	/**
   	 * Width, in pixels, of the chart. If the container div has been explicitly sized, this will be ignored.
   	 *
   	 * @var integer
   	 */
   	public $width;
   	
   	/**
   	 * TODO(konigsberg): document this
   	 *
   	 * @var Object
   	 */
   	public $interactionModel;
   	
   	/**
   	 * This lets you specify an arbitrary function to generate tick marks on an axis. The tick marks are an array of (value, label) pairs. The built-in functions go to great lengths to choose good tick marks so, if you set this option, you'll most likely want to call one of them and modify the result. See dygraph-tickers.js for an extensive discussion. This is set on a <a href='per-axis.html'>per-axis</a> basis.
   	 *
   	 * @var function(min, max, pixels, opts, dygraph, vals) -> [{v: ..., label: ...}, ...]
   	 */
   	public $ticker;
   	
   	/**
   	 * Height, in pixels, of the x-axis. If not set explicitly, this is computed based on axisLabelFontSize and axisTickSize.
   	 *
   	 * @var integer
   	 */
   	public $xAxisHeight;
   	
   	/**
   	 * Whether to show the legend upon mouseover.
   	 *
   	 * @var boolean
   	 */
   	public $showLabelsOnHighlight;
   	
   	/**
   	 * Set to either 'y1' or 'y2' to assign a series to a y-axis (primary or secondary). Must be set per-series.
   	 *
   	 * @var string
   	 */
   	public $axis;
   	
   	/**
   	 * Number of pixels to require between each x- and y-label. Larger values will yield a sparser axis with fewer ticks. This is set on a <a href='per-axis.html'>per-axis</a> basis.
   	 *
   	 * @var integer
   	 */
   	public $pixelsPerLabel;
   	
   	/**
   	 * Show data labels in an external div, rather than on the graph.  This value can either be a div element or a div id.
   	 *
   	 * @var DOM element or string
   	 */
   	public $labelsDiv;
   	
   	/**
   	 * When set, attempt to parse each cell in the CSV file as "a/b", where a and b are integers. The ratio will be plotted. This allows computation of Wilson confidence intervals (see below).
   	 *
   	 * @var boolean
   	 */
   	public $fractions;
   	
   	/**
   	 * When set for the y-axis or x-axis, the graph shows that axis in log scale. Any values less than or equal to zero are not displayed. Showing log scale with ranges that go below zero will result in an unviewable graph.
   	
   	 Not compatible with showZero. connectSeparatedPoints is ignored. This is ignored for date-based x-axes.
   	 *
   	 * @var boolean
   	 */
   	public $logscale;
   	
   	/**
   	 * The width of the lines connecting data points. This can be used to increase the contrast or some graphs.
   	 *
   	 * @var float
   	 */
   	public $strokeWidth;
   	
   	/**
   	 * A custom pattern array where the even index is a draw and odd is a space in pixels. If null then it draws a solid line. The array should have a even length as any odd lengthed array could be expressed as a smaller even length array. This is used to create dashed lines.
   	 *
   	 * @var array<integer>
   	 */
   	public $strokePattern;
   	
   	/**
   	 * Draw a border around graph lines to make crossing lines more easily distinguishable. Useful for graphs with many lines.
   	 *
   	 * @var float
   	 */
   	public $strokeBorderWidth;
   	
   	/**
   	 * Color for the line border used if strokeBorderWidth is set.
   	 *
   	 * @var string
   	 */
   	public $strokeBorderColor;
   	
   	/**
   	 * Use in conjunction with the "fractions" option. Instead of plotting +/- N standard deviations, dygraphs will compute a Wilson confidence interval and plot that. This has more reasonable behavior for ratios close to 0 or 1.
   	 *
   	 * @var boolean
   	 */
   	public $wilsonInterval;
   	
   	/**
   	 * Should the area underneath the graph be filled? This option is not compatible with error bars. This may be set on a <a href='per-axis.html'>per-series</a> basis.
   	 *
   	 * @var boolean
   	 */
   	public $fillGraph;
   	
   	/**
   	 * The size in pixels of the dot drawn over highlighted points.
   	 *
   	 * @var integer
   	 */
   	public $highlightCircleSize;
   	
   	/**
   	 * The color of the gridlines. This may be set on a per-axis basis to define each axis' grid separately.
   	 *
   	 * @var red, blue
   	 */
   	public $gridLineColor;
   	
   	/**
   	 * A custom pattern array where the even index is a draw and odd is a space in pixels. If null then it draws a solid line. The array should have a even length as any odd lengthed array could be expressed as a smaller even length array. This is used to create dashed gridlines.
   	 *
   	 * @var array<integer>
   	 */
   	public $gridLinePattern;
   	
   	/**
   	 * Which series should initially be visible? Once the Dygraph has been constructed, you can access and modify the visibility of each series using the <code>visibility</code> and <code>setVisibility</code> methods.
   	 *
   	 * @var Array of booleans
   	 */
   	public $visibility;
   	
   	/**
   	 * Explicitly set the vertical range of the graph to [low, high]. This may be set on a per-axis basis to define each y-axis separately. If either limit is unspecified, it will be calculated automatically (e.g. [null, 30] to automatically calculate just the lower bound)
   	 *
   	 * @var Array of two numbers
   	 */
   	public $valueRange;
   	
   	/**
   	 * If <strong>colors</strong> is not specified, saturation of the automatically-generated data series colors.
   	 *
   	 * @var float (0.0 - 1.0)
   	 */
   	public $colorSaturation;
   	
   	/**
   	 * Whether to hide the legend when the mouse leaves the chart area.
   	 *
   	 * @var boolean
   	 */
   	public $hideOverlayOnMouseOut;
   	
   	/**
   	 * When to display the legend. By default, it only appears when a user mouses over the chart. Set it to "always" to always display a legend of some sort. When set to "follow", legend follows highlighted points.
   	 *
   	 * @var string
   	 */
   	public $legend;
   	
   	/**
   	 * Set this to supply a custom formatter for the legend. See <a href="https://github.com/danvk/dygraphs/pull/683">this comment</a> and the <a href="tests/legend-formatter.html">legendFormatter demo</a> for usage.
   	 *
   	 * @var function(data): string
   	 */
   	public $legendFormatter;
   	
   	/**
   	 * Show zero value labels in the labelsDiv.
   	 *
   	 * @var boolean
   	 */
   	public $labelsShowZeroValues;
   	
   	/**
   	 * When set, display the graph as a step plot instead of a line plot. This option may either be set for the whole graph or for single series.
   	 *
   	 * @var boolean
   	 */
   	public $stepPlot;
   	
   	/**
   	 * Show date/time labels according to UTC (instead of local time).
   	 *
   	 * @var boolean
   	 */
   	public $labelsUTC;
   	
   	/**
   	 * Show K/M/B for thousands/millions/billions on y-axis.
   	 *
   	 * @var boolean
   	 */
   	public $labelsKMB;
   	
   	/**
   	 * Number of pixels to leave blank at the right edge of the Dygraph. This makes it easier to highlight the right-most data point.
   	 *
   	 * @var integer
   	 */
   	public $rightGap;
   	
   	/**
   	 * Deprecated, please use yRangePad instead. When set, the heuristic that fixes the Y axis at zero for a data set with the minimum Y value of zero is disabled.
   	 This is particularly useful for data sets that contain many zero values, especially for step plots which may otherwise have lines not visible running along the bottom axis.
   	 *
   	 * @var boolean
   	 */
   	public $avoidMinZero;
   	
   	/**
   	 * When set, draw the X axis at the Y=0 position and the Y axis at the X=0 position if those positions are inside the graph's visible area. Otherwise, draw the axes at the bottom or left graph edge as usual.
   	 *
   	 * @var boolean
   	 */
   	public $drawAxesAtZero;
   	
   	/**
   	 * Add the specified amount of extra space (in pixels) around the X-axis value range to ensure points at the edges remain visible.
   	 *
   	 * @var float
   	 */
   	public $xRangePad;
   	
   	/**
   	 * If set, add the specified amount of extra space (in pixels) around the Y-axis value range to ensure points at the edges remain visible. If unset, use the traditional Y padding algorithm.
   	 *
   	 * @var float
   	 */
   	public $yRangePad;
   	
   	/**
   	 * Function to call to format the tick values that appear along an axis. This is usually set on a <a href='per-axis.html'>per-axis</a> basis.
   	 *
   	 * @var function(number or Date, granularity, opts, dygraph)
   	 */
   	public $axisLabelFormatter;
   	
   	/**
   	 * A function to call when the canvas is clicked.
   	 *
   	 * @var function(e, x, points)
   	 */
   	public $clickCallback;
   	
   	/**
   	 * A name for each data series, including the independent (X) series. For CSV files and DataTable objections, this is determined by context. For raw data, this must be specified. If it is not, default values are supplied and a warning is logged.
   	 *
   	 * @var array<string>
   	 */
   	public $labels;
   	
   	/**
   	 * Initially zoom in on a section of the graph. Is of the form [earliest, latest], where earliest/latest are milliseconds since epoch. If the data for the x-axis is numeric, the values in dateWindow must also be numbers.
   	 *
   	 * @var Array of two numbers
   	 */
   	public $dateWindow;
   	
   	/**
   	 * If the rolling average period text box should be shown.
   	 *
   	 * @var boolean
   	 */
   	public $showRoller;
   	
   	/**
   	 * When errorBars is set, shade this many standard deviations above/below each point.
   	 *
   	 * @var float
   	 */
   	public $sigma;
   	
   	/**
   	 * When set, parse each CSV cell as "low;middle;high". Error bars will be drawn for each point between low and high, with the series itself going through middle.
   	 *
   	 * @var boolean
   	 */
   	public $customBars;
   	
   	/**
   	 * If colors is not specified, value of the data series colors, as in hue/saturation/value. (0.0-1.0, default 0.5)
   	 *
   	 * @var float (0.0 - 1.0)
   	 */
   	public $colorValue;
   	
   	/**
   	 * Does the data contain standard deviations? Setting this to true alters the input format (see above).
   	 *
   	 * @var boolean
   	 */
   	public $errorBars;
   	
   	/**
   	 * Only applies when Dygraphs is used as a GViz chart. Causes string columns following a data series to be interpreted as annotations on points in that series. This is the same format used by Google's AnnotatedTimeLine chart.
   	 *
   	 * @var boolean
   	 */
   	public $displayAnnotations;
   	
   	/**
   	 * A value representing the farthest a graph may be panned, in percent of the display. For example, a value of 0.1 means that the graph can only be panned 10% pased the edges of the displayed values. null means no bounds.
   	 *
   	 * @var float
   	 */
   	public $panEdgeFraction;
   	
   	/**
   	 * Text to display above the chart. You can supply any HTML for this value, not just text. If you wish to style it using CSS, use the 'dygraph-label' or 'dygraph-title' classes.
   	 *
   	 * @var string
   	 */
   	public $title;
   	
   	/**
   	 * Height of the chart title, in pixels. This also controls the default font size of the title. If you style the title on your own, this controls how much space is set aside above the chart for the title's div.
   	 *
   	 * @var integer
   	 */
   	public $titleHeight;
   	
   	/**
   	 * Text to display below the chart's x-axis. You can supply any HTML for this value, not just text. If you wish to style it using CSS, use the 'dygraph-label' or 'dygraph-xlabel' classes.
   	 *
   	 * @var string
   	 */
   	public $xlabel;
   	
   	/**
   	 * Height of the x-axis label, in pixels. This also controls the default font size of the x-axis label. If you style the label on your own, this controls how much space is set aside below the chart for the x-axis label's div.
   	 *
   	 * @var integer
   	 */
   	public $xLabelHeight;
   	
   	/**
   	 * Text to display to the left of the chart's y-axis. You can supply any HTML for this value, not just text. If you wish to style it using CSS, use the 'dygraph-label' or 'dygraph-ylabel' classes. The text will be rotated 90 degrees by default, so CSS rules may behave in unintuitive ways. No additional space is set aside for a y-axis label. If you need more space, increase the width of the y-axis tick labels using the yAxisLabelWidth option. If you need a wider div for the y-axis label, either style it that way with CSS (but remember that it's rotated, so width is controlled by the 'height' property) or set the yLabelWidth option.
   	 *
   	 * @var string
   	 */
   	public $ylabel;
   	
   	/**
   	 * Text to display to the right of the chart's secondary y-axis. This label is only displayed if a secondary y-axis is present. See <a href='http://dygraphs.com/tests/two-axes.html'>this test</a> for an example of how to do this. The comments for the 'ylabel' option generally apply here as well. This label gets a 'dygraph-y2label' instead of a 'dygraph-ylabel' class.
   	 *
   	 * @var string
   	 */
   	public $y2label;
   	
   	/**
   	 * Width of the div which contains the y-axis label. Since the y-axis label appears rotated 90 degrees, this actually affects the height of its div.
   	 *
   	 * @var integer
   	 */
   	public $yLabelWidth;
   	
   	/**
   	 * When this option is passed to updateOptions() along with either the <code>dateWindow</code> or <code>valueRange</code> options, the zoom flags are not changed to reflect a zoomed state. This is primarily useful for when the display area of a chart is changed programmatically and also where manual zooming is allowed and use is made of the <code>isZoomed</code> method to determine this.
   	 *
   	 * @var boolean
   	 */
   	public $isZoomedIgnoreProgrammaticZoom;
   	
   	/**
   	 * Whether to display gridlines in the chart. This may be set on a per-axis basis to define the visibility of each axis' grid separately.
   	 *
   	 * @var boolean
   	 */
   	public $drawGrid;
   	
   	/**
   	 * Only valid for y and y2, has no effect on x: This option defines whether the y axes should align their ticks or if they should be independent. Possible combinations: 1.) y=true, y2=false (default): y is the primary axis and the y2 ticks are aligned to the the ones of y. (only 1 grid) 2.) y=false, y2=true: y2 is the primary axis and the y ticks are aligned to the the ones of y2. (only 1 grid) 3.) y=true, y2=true: Both axis are independent and have their own ticks. (2 grids) 4.) y=false, y2=false: Invalid configuration causes an error.
   	 *
   	 * @var boolean
   	 */
   	public $independentTicks;
   	
   	/**
   	 * Whether to draw the specified axis. This may be set on a per-axis basis to define the visibility of each axis separately. Setting this to false also prevents axis ticks from being drawn and reclaims the space for the chart grid/lines.
   	 *
   	 * @var boolean
   	 */
   	public $drawAxis;
   	
   	/**
   	 * Thickness (in pixels) of the gridlines drawn under the chart. The vertical/horizontal gridlines can be turned off entirely by using the drawGrid option. This may be set on a per-axis basis to define each axis' grid separately.
   	 *
   	 * @var float
   	 */
   	public $gridLineWidth;
   	
   	/**
   	 * Thickness (in pixels) of the x- and y-axis lines.
   	 *
   	 * @var float
   	 */
   	public $axisLineWidth;
   	
   	/**
   	 * Color of the x- and y-axis lines. Accepts any value which the HTML canvas strokeStyle attribute understands, e.g. 'black' or 'rgb(0, 100, 255)'.
   	 *
   	 * @var string
   	 */
   	public $axisLineColor;
   	
   	/**
   	 * Error bars (or custom bars) for each series are drawn in the same color as the series, but with partial transparency. This sets the transparency. A value of 0.0 means that the error bars will not be drawn, whereas a value of 1.0 means that the error bars will be as dark as the line for the series itself. This can be used to produce chart lines whose thickness varies at each point.
   	 *
   	 * @var float (0.0 - 1.0)
   	 */
   	public $fillAlpha;
   	
   	/**
   	 * Width (in pixels) of the containing divs for x- and y-axis labels. For the y-axis, this also controls the width of the y-axis. Note that for the x-axis, this is independent from pixelsPerLabel, which controls the spacing between labels.
   	 *
   	 * @var integer
   	 */
   	public $axisLabelWidth;
   	
   	/**
   	 * By default, dygraphs displays numbers with a fixed number of digits after the decimal point. If you'd prefer to have a fixed number of significant figures, set this option to that number of sig figs. A value of 2, for instance, would cause 1 to be display as 1.0 and 1234 to be displayed as 1.23e+3.
   	 *
   	 * @var integer
   	 */
   	public $sigFigs;
   	
   	/**
   	 * Unless it's run in scientific mode (see the <code>sigFigs</code> option), dygraphs displays numbers with <code>digitsAfterDecimal</code> digits after the decimal point. Trailing zeros are not displayed, so with a value of 2 you'll get '0', '0.1', '0.12', '123.45' but not '123.456' (it will be rounded to '123.46'). Numbers with absolute value less than 0.1^digitsAfterDecimal (i.e. those which would show up as '0.00') will be displayed in scientific notation.
   	 *
   	 * @var integer
   	 */
   	public $digitsAfterDecimal;
   	
   	/**
   	 * When displaying numbers in normal (not scientific) mode, large numbers will be displayed with many trailing zeros (e.g. 100000000 instead of 1e9). This can lead to unwieldy y-axis labels. If there are more than <code>maxNumberWidth</code> digits to the left of the decimal in a number, dygraphs will switch to scientific notation, even when not operating in scientific mode. If you'd like to see all those digits, set this to something large, like 20 or 30.
   	 *
   	 * @var integer
   	 */
   	public $maxNumberWidth;
   	
   	/**
   	 * Sets the data being displayed in the chart. This can only be set when calling updateOptions; it cannot be set from the constructor. For a full description of valid data formats, see the <a href='http://dygraphs.com/data.html'>Data Formats</a> page.
   	 *
   	 * @var string (URL of CSV or CSV), GViz DataTable or 2D Array
   	 */
   	public $file;
   	
   	/**
   	 * Set this option to log timing information. The value of the option will be logged along with the timimg, so that you can distinguish multiple dygraphs on the same page.
   	 *
   	 * @var string
   	 */
   	public $timingName;
   	
   	/**
   	 * Show or hide the range selector widget.
   	 *
   	 * @var boolean
   	 */
   	public $showRangeSelector;
   	
   	/**
   	 * Height, in pixels, of the range selector widget. This option can only be specified at Dygraph creation time.
   	 *
   	 * @var integer
   	 */
   	public $rangeSelectorHeight;
   	
   	/**
   	 * The range selector mini plot stroke color. This can be of the form "#AABBCC" or "rgb(255,100,200)" or "yellow". You can also specify null or "" to turn off stroke.
   	 *
   	 * @var string
   	 */
   	public $rangeSelectorPlotStrokeColor;
   	
   	/**
   	 * The range selector mini plot fill color. This can be of the form "#AABBCC" or "rgb(255,100,200)" or "yellow". You can also specify null or "" to turn off fill.
   	 *
   	 * @var string
   	 */
   	public $rangeSelectorPlotFillColor;
   	
   	/**
   	 * The top color for the range selector mini plot fill color gradient. This can be of the form "#AABBCC" or "rgb(255,100,200)" or "rgba(255,100,200,42)" or "yellow". You can also specify null or "" to disable the gradient and fill with one single color.
   	 *
   	 * @var string
   	 */
   	public $rangeSelectorPlotFillGradientColor;
   	
   	/**
   	 * The color of the lines below and on both sides of the range selector mini plot. This can be of the form "#AABBCC" or "rgb(255,100,200)" or "yellow".
   	 *
   	 * @var string
   	 */
   	public $rangeSelectorBackgroundStrokeColor;
   	
   	/**
   	 * The width of the lines below and on both sides of the range selector mini plot.
   	 *
   	 * @var float
   	 */
   	public $rangeSelectorBackgroundLineWidth;
   	
   	/**
   	 * The width of the range selector mini plot line.
   	 *
   	 * @var float
   	 */
   	public $rangeSelectorPlotLineWidth;
   	
   	/**
   	 * The color of the lines in the interactive layer of the range selector. This can be of the form "#AABBCC" or "rgb(255,100,200)" or "yellow".
   	 *
   	 * @var string
   	 */
   	public $rangeSelectorForegroundStrokeColor;
   	
   	/**
   	 * The width the lines in the interactive layer of the range selector.
   	 *
   	 * @var float
   	 */
   	public $rangeSelectorForegroundLineWidth;
   	
   	/**
   	 * The transparency of the veil that is drawn over the unselected portions of the range selector mini plot. A value of 0 represents full transparency and the unselected portions of the mini plot will appear as normal. A value of 1 represents full opacity and the unselected portions of the mini plot will be hidden.
   	 *
   	 * @var float (0.0 - 1.0)
   	 */
   	public $rangeSelectorAlpha;
   	
   	/**
   	 * Mark this series for inclusion in the range selector. The mini plot curve will be an average of all such series. If this is not specified for any series, the default behavior is to average all the visible series. Setting it for one series will result in that series being charted alone in the range selector. Once it's set for a single series, it needs to be set for all series which should be included (regardless of visibility).
   	 *
   	 * @var boolean
   	 */
   	public $showInRangeSelector;
   	
   	/**
   	 * Set this option to animate the transition between zoom windows. Applies to programmatic and interactive zooms. Note that if you also set a drawCallback, it will be called several times on each zoom. If you set a zoomCallback, it will only be called after the animation is complete.
   	 *
   	 * @var boolean
   	 */
   	public $animatedZooms;
   	
   	/**
   	 * A function (or array of functions) which plot each data series on the chart. TODO(danvk): more details! May be set per-series.
   	 *
   	 * @var array or function
   	 */
   	public $plotter;
   	
   	/**
   	 * Defines per-axis options. Valid keys are 'x', 'y' and 'y2'. Only some options may be set on a per-axis basis. If an option may be set in this way, it will be noted on this page. See also documentation on <a href='http://dygraphs.com/per-axis.html'>per-series and per-axis options</a>.
   	 *
   	 * @var Object
   	 */
   	public $axes;
   	
   	/**
   	 * Defines per-series options. Its keys match the y-axis label names, and the values are dictionaries themselves that contain options specific to that series.
   	 *
   	 * @var Object
   	 */
   	public $series;
   	
   	/**
   	 * Defines per-graph plugins. Useful for per-graph customization
   	 *
   	 * @var Array<plugin>
   	 */
   	public $plugins;
   	
   	/**
   	 * Custom DataHandler. This is an advanced customization. See http://bit.ly/151E7Aq.
   	 *
   	 * @var Dygraph.DataHandler
   	 */
   	public $dataHandler;
}